public with sharing class TestAuditService {
    
    public void testAuditService(){
    	String packageId = 'NU1klEhxdLOm4FVPf-WFfh1a9_I=';
    	List<TestAuditService.Audit> auditList = getAudit(packageId);
    	for(TestAuditService.Audit audit: auditList){
    		System.debug('DateTime - ' + audit.date_time + ', Type - ' + audit.type + ', User - ' +  audit.user + ', Email -' + audit.user_email + ', IP - ' + audit.user_ip + ', Target - ' + audit.target + ', Data - ' + audit.data);
    	}
    }
    
    
    
 
 public List<TestAuditService.Audit> getAudit(String packageId){
 	      // Send Request
        String resourceUrl = '/packages/'+packageId+'/audit';
        HttpResponse response = doGetWithEnhancedInboundJSON(resourceUrl);

        // Check response
        if(response.getStatusCode() == 200)
        {
            try
            {
            	Map<String,Object> responseObject = (Map<String,Object>)JSON.deserializeUntyped(response.getBody());
            	List<Object> auditObjects = (List<Object>)responseObject.get('audit-events');
            	List<TestAuditService.Audit> audits = new List<TestAuditService.Audit>();
            	for(Object auditObject : auditObjects){
            		audits.add((TestAuditService.Audit)JSON.deserialize(JSON.serialize(auditObject),TestAuditService.Audit.class));
            	}
            	return audits;
            }
            catch(Exception e)
            {
                throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error converting audit json ' + e.getMessage());
            }
        }   
        else
        {
            throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error get Audit ' + response.getStatusCode() + ' - ' + response.getStatus() + ' - ' +  response.getBody());
        }
 }
    
    
    
    
    private HttpResponse doGetWithEnhancedInboundJSON(String resourceUrl)
    {
    	e_SignLive_Connection_Settings__c connectionSettings = e_SignLive_Connection_Settings__c.getInstance('Main');    
        // Build Request
        HttpRequest request = new HttpRequest();
        request.setEndpoint(connectionSettings.Endpoint__c + resourceUrl);
        request.setMethod('GET');
        request.setHeader('Authorization', 'Basic ' + connectionSettings.API_Key__c);
        request.setHeader('Content-type', 'application/json');

        // Send Request
        HttpResponse response = (new Http()).send(request);

        if(response != null && (response.getHeader('Content-Type')).contains('json'))
        {
            System.debug('=> Formating for JSON response');
            // Format responseBody string before returning
            String inboundResponseBody = prepareEnhancedInboundJSON(response.getBody());
            response.setBody(inboundResponseBody);
        }

        return response;
    }
    
    
     private String prepareEnhancedInboundJSON(String jsonString)
    {
        System.debug('=> prepareOutboundJSON() param ' + jsonString);
        if(String.isNotEmpty(jsonString))
        {
            Map<String, String> inboundReplacementMap = getInboundReplacementMap();
            for(String key : inboundReplacementMap.keySet())
            {
                if(jsonString.contains(key))
                {
                    jsonString = jsonString.replace(key, inboundReplacementMap.get(key));
                }
            }
            
            System.debug('inbound response body: ' + jsonString);
        }
        jsonString = removeNullProperties(jsonString);
        return jsonString;
    }
    
    
    private  Map<String, String> getInboundReplacementMap()
    {
        Map<String, String> inboundReplacementMap = new Map<String, String>();
        inboundReplacementMap.put('"enum"', '"enum_x"');
        inboundReplacementMap.put('"from"', '"from_x"');
        inboundReplacementMap.put('"group"', '"group_x"');
        inboundReplacementMap.put('NEW', 'NEW_X');
        inboundReplacementMap.put('PACKAGE', 'PACKAGE_X');
        inboundReplacementMap.put('PACKAGE_X_X', 'PACKAGE_X');
		inboundReplacementMap.put('date-time','date_time');	//added line
		inboundReplacementMap.put('user-email','user_email');	//added line
		inboundReplacementMap.put('user-ip','user_ip');	//added line
        return inboundReplacementMap;
    }
    
    private String removeNullProperties(String jsonString)
    {
        if(String.isNotEmpty(jsonString))       
        {
            jsonString = jsonString.replaceAll('\"[^\"]*\" *: *null *, *','');
            jsonString = jsonString.replaceAll('\"[^\"]*\" *: *null *','');
            jsonString = jsonString.replaceAll(',{2,}', ',');
            jsonString = jsonString.replace('{,', '{');
            jsonString = jsonString.replace(',}', '}');
            jsonString = jsonString.replace('[,', '[');
            jsonString = jsonString.replace(',]', ']');
        }
    
        return jsonString;
    }
    
    
      public class Audit
    {
        public String type;
        public String date_time;
        public String target;
        public String target_type;
        public String user;
        public String user_email;
        public String user_ip;
        public String data;

        public Audit(){}

        public Audit(String type,String date_time,String target,String target_type,String user,String user_email, String user_ip,String data)
        {
            this.type = type;
            this.date_time = date_time;
            this.target = target;
            this.target_type = target_type;
            this.user = user;
            this.user_email = user_email;
            this.user_ip = user_ip;
            this.data = data;
        }
    }
    
}